
public class SongAnalysis {

    /*
     * PRELIMINARIES!
     * readData, readSong, and readSongs are functions written for you
     * to enable you to run & test your code on your own data!
     *
     * First, you should modify the array stored in colsToInclude in
     * the readSong function so that it refers to all column indices
     * that must be included based on the Song record definition.
     * This is the ONLY modification you need to make to the first
     * three functions in this file.
     *
     * Your task will be to write other functions in Sections A, B, & C below.
     */

    /*
     * Input: an In object set to read from a given CSV
     * Output: a String containing the data in next cell of
     * the CSV.
     */
    public static String readData(In inStream) {
        String data = "";
        boolean insideQuotes = false;
        boolean afterQuotes = false;

        // stop if there's no text in the file (bad!)
        // or if we have read a comma after two "s
        while (!inStream.isEmpty() && !afterQuotes) {
            char currentChar = inStream.readChar();
            if (currentChar == '"') {
                // Handle opening and closing quotes
                insideQuotes = !insideQuotes;
            } else if (currentChar == ',' && !insideQuotes) {
                // Found the comma outside of quotes
                afterQuotes = true;
            } else {
                // Append the character to the data
                data += currentChar;
            }
        }
        return data;
    }

    /*
     * Input: an In object set to read from a given CSV
     * Output: a Song record containing all of the data
     * for the next song to be read from the CSV
     * TODO: Change first line so that colsToInclude is an array
     * containing all of the relevant column indices.
     */
    public static Song readSong(In inStream) {
        // TODO: specify the correct indices of the 10 columns to include
        int[] colsToInclude = { 2, 3 };
        String[] newRow = new String[colsToInclude.length];
        int colsRead = 0;
        for (int i = 0; colsRead < colsToInclude.length; i++) {
            String data = readData(inStream);
            if (i == colsToInclude[colsRead]) {
                newRow[colsRead] = data;
                colsRead++;
            }
        }
        inStream.readLine(); // skip to the next line
        /*
         * String properties can be copied directly.
         * 
         * Properties like artist & genre need to be
         * turned into arrays, so we separate the Strings
         * into String[] by calling split()
         * 
         * Numeric properties need to be parsed as ints
         * and doubles using the corresponding X.parseX()
         * functions.
         */
        return new Song(newRow[0], newRow[1].split(", "),
                newRow[2], newRow[3],
                Integer.parseInt(newRow[4]),
                Integer.parseInt(newRow[5]),
                newRow[6].split(","),
                Double.parseDouble(newRow[7]),
                Double.parseDouble(newRow[8]),
                Double.parseDouble(newRow[9]));

    }

    /*
     * Input: an In object set to read from a given CSV
     * Output: an array of Song records, one for each row
     * of the input file.
     */
    public static Song[] readSongs(String filename) {

        // start by counting the songs
        In libraryReader = new In(filename);
        libraryReader.readLine(); // skip the header row
        int numSongs = 0;
        while (!libraryReader.isEmpty()) {
            libraryReader.readLine();
            numSongs++;
        }

        // restart and read the songs this time.
        libraryReader = new In(filename);
        libraryReader.readLine(); // skip the header row

        Song[] songs = new Song[numSongs];
        for (int i = 0; i < numSongs; i++) {
            songs[i] = readSong(libraryReader);
        }

        return songs;

    }

    /*
     * Feel free to delete or modify main as much as you
     * need to run & test your code. We will not use it
     * when testing your program.
     */
    public static void main(String[] args) {
        // by default, main will read all songs in the
        // file provided as a command line argument
        // and print out one in every 50 songs.
        String filename = args[0];
        Song[] library = readSongs(filename);
        for (int i = 0; i < library.length; i += 50) {
            System.out.println(library[i]);
        }
    }

    /*
     * SECTION A: Complete two out of these three functions.
     * If you complete all three, only the first two will be
     * graded. There is no extra credit.
     */

    /**
     * Counts the number of songs from a specific year in the library.
     * If the library is empty, return 0.
     * Inputs:
     * library An array of Song objects.
     * year The year to count songs from.
     * Output: The number of songs from the specified year.
     */
    public static int countSongsFromYear(Song[] library, int year) {
        return 0;
    }

    /**
     * Finds the song with the most genres in the library and
     * returns its name. Return the empty String ("") if the library
     * is empty.
     * Inputs:
     * library An array of Song objects.
     * Outputs:
     * The track name of the song with the most genres, or
     * the empty String if the library is empty.
     */
    public static String songNameWithMostGenres(Song[] library) {
        return null;
    }

    /**
     * Calculates the range of popularity in the library.
     * Ignores all songs with popularity 0. If there is no
     * song in the library, return 0.
     *
     * Inputs:
     * library An array of Song objects.
     * Outputs:
     * The range of popularity values (highest popularity - lowest popularity)
     */
    public static int calculatePopularityRange(Song[] library) {
        return 0;

    }

    /*
     * SECTION B: Complete two out of these three functions.
     * If you complete all three, only the first two will be
     * graded. There is no extra credit.
     */

    /**
     * Selects all songs by a specific artist in the library.
     * Keep in mind that a song may have many artists. The song
     * should be included in the output if any of its artists
     * match the input.
     *
     * Inputs:
     * library An array of Song objects.
     * artist The name of the artist to search for.
     * Outputs:
     * An array of Song objects by the specified artist.
     */
    public static Song[] findSongsWithArtist(Song[] library, String artist) {
        return null;
    }

    /**
     * Calculates the standard deviation of energy levels in the library.
     * The standard deviation is defined as the square root of the average
     * squared difference from the average.
     *
     * Inputs:
     * library An array of Song objects.
     * Outputs:
     * The standard deviation of energy levels.
     */
    public static double standardDeviationOfEnergy(Song[] library) {
        return 0;
    }

    /**
     * Returns an array containing the longest songs by decade.
     * Decades are defined starting as 1960-1969, 1970-1979, etc.,
     * up until 2020-2029. (Of course, there are no songs after 2023...)
     * There are seven decades all told, so the output array should contain
     * seven Song records. If there is no Song present in a given decade,
     * the array should contain a null value in the corresponding position.
     * 
     * Inputs:
     * library, an array of Song objects.
     *
     * Output:
     * An array containing seven Song records representing the longest
     * tracks per decade.
     */
    public static Song[] longestTrackByDecade(Song[] library) {
        // 1960s is decade 0, 1970s is decade 1, 1980s is decade 2,
        // 1990s is decade 3, 2000s is decade 4, 2010s is decade 5,
        // 2020s is decade 6

        return null;
    }

    /*
     * SECTION C: You must complete this function.
     */

    /*
     * Draw a scatter plot that contains one point per song.
     * The x-axis should represent the danceability of
     * the song. The y-axis should represent the energy of the
     * song. Each dot should be drawn with a pen radius that increases
     * with the loudness of the Song. A song with a loudness of -60 should
     * be drawn with a pen radius of 0.001. A song with a loudness of 0
     * (very loud) should be drawn with a pen radius of 0.015.
     * All loudnessess in between should scale linearly (-30 is halfway between so
     * it would be drawn with a radius of 0.008).
     */
    public static void scatterPlot(Song[] library) {

    }
}