import static org.junit.Assert.*;
import org.junit.*;

public class SongAnalysisTest {

    @Test
    public void countSongsFromYearSmall2015() {
        Song[] library = SongAnalysis.readSongs("small.csv");
        int year = 2015;

        int expected = 4;
        int actual = SongAnalysis.countSongsFromYear(library, year);
        assertEquals("There should be four songs from 2015 in small.csv",
                expected, actual);
    }

    @Test
    public void countSongsFromYearHarry2020() {
        Song[] library = SongAnalysis.readSongs("sharry_songs.csv");
        int year = 2020;

        int expected = 78;
        int actual = SongAnalysis.countSongsFromYear(library, year);
        assertEquals("There should be 78 songs from 2020 in sharry_songs.csv",
                expected, actual);
    }

    @Test
    public void songNameWithMostGenresMedium() {
        Song[] library = SongAnalysis.readSongs("medium.csv");

        String expected = "american dream";
        String actual = SongAnalysis.songNameWithMostGenres(library);
        assertEquals(expected, actual);
    }

    @Test
    public void songNameWithMostGenresHarry() {
        Song[] library = SongAnalysis.readSongs("sharry_songs.csv");

        String expected = "...And The World Laughs With You";
        String actual = SongAnalysis.songNameWithMostGenres(library);
        assertEquals(expected, actual);
    }

    @Test
    public void calculatePopularityRangeSmall() {
        Song[] library = SongAnalysis.readSongs("small.csv");

        int expected = 29;
        int actual = SongAnalysis.calculatePopularityRange(library);
        assertEquals(expected, actual);
    }

    @Test
    public void calculatePopularityRangeHarry() {
        Song[] library = SongAnalysis.readSongs("sharry_songs.csv");

        int expected = 92;
        int actual = SongAnalysis.calculatePopularityRange(library);
        assertEquals(expected, actual);
    }

    @Test
    public void findSongsWithArtistMedium() {
        Song[] library = SongAnalysis.readSongs("medium.csv");
        String artist = "Slash";

        String[] expected = { "Starlight (feat. Myles Kennedy)", "Watch This" };
        Song[] actualSongs = SongAnalysis.findSongsWithArtist(library, artist);
        String[] actual = songsToNames(actualSongs);
        assertArrayEquals(expected, actual);
    }

    @Test
    public void findSongsWithArtistHarry() {
        Song[] library = SongAnalysis.readSongs("sharry_songs.csv");
        String artist = "Phoebe Bridgers";

        String expectedFirst = "Leonard Cohen";
        String expectedSecond = "Motion Sickness";
        String expectedLast = "Dominos";
        int expectedLength = 13;

        Song[] actualSongs = SongAnalysis.findSongsWithArtist(library, artist);
        int actualLength = actualSongs.length;
        assertTrue(containsSongWithName(actualSongs, expectedFirst));
        assertTrue(containsSongWithName(actualSongs, expectedSecond));
        assertTrue(containsSongWithName(actualSongs, expectedLast));
        assertEquals(expectedLength, actualLength);
    }

    @Test
    public void standardDeviationOfEnergyMedium() {
        Song[] library = SongAnalysis.readSongs("medium.csv");

        double expected = 0.23704089191222488;
        double actual = SongAnalysis.standardDeviationOfEnergy(library);
        assertEquals(expected, actual, 0.0001);
    }

    @Test
    public void standardDeviationOfEnergyHarry() {
        Song[] library = SongAnalysis.readSongs("sharry_songs.csv");

        double expected = 0.2304657392650899;
        double actual = SongAnalysis.standardDeviationOfEnergy(library);
        assertEquals(expected, actual, 0.0001);
    }

    @Test
    public void longestTrackByDecadeSmall() {
        Song[] library = SongAnalysis.readSongs("small.csv");

        String[] expected = { null, null, null,
                "Pet Politics", "Couldn't Stand the Weather",
                "Fly", null };

        Song[] actualSongs = SongAnalysis.longestTrackByDecade(library);
        String[] actual = songsToNames(actualSongs);
        assertArrayEquals(expected, actual);
    }

    @Test
    public void longestTrackByDecadeHarry() {
        Song[] library = SongAnalysis.readSongs("sharry_songs.csv");

        String[] expected = { "Desolation Row - Alternate Take",
                "Europe Endless - 2009 Remaster",
                "To Live Is To Die", "Opium",
                "240 Years Before Your Time",
                "The Battle Of Hampton Roads",
                "Doomscroller" };

        Song[] actualSongs = SongAnalysis.longestTrackByDecade(library);
        String[] actual = songsToNames(actualSongs);
        assertArrayEquals(expected, actual);
    }

    public static boolean containsSongWithName(Song[] output, String name) {
        for (int i = 0; i < output.length; i++) {
            if (output[i].trackName().equals(name)) {
                return true;
            }
        }
        return false;
    }

    public static String[] songsToNames(Song[] songs) {
        String[] names = new String[songs.length];
        for (int i = 0; i < songs.length; i++) {
            if (songs[i] != null) {
                names[i] = songs[i].trackName();
            }
        }
        return names;
    }

}